!
! Copyright (C) 2001-2009 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!--------------------------------------------------------------------
SUBROUTINE dos_sub()
  !--------------------------------------------------------------------
  !
  ! Calculates the Density of States (DOS),
  ! separated into up and down components for LSDA
  !
  USE io_global,  ONLY : stdout, ionode, ionode_id
  USE io_files,   ONLY : prefix, tmp_dir
  USE constants,  ONLY : rytoev
  USE kinds,      ONLY : DP
  USE klist,      ONLY : xk, wk, degauss, ngauss, lgauss, nks, nkstot
  USE ktetra,     ONLY : ntetra, tetra, ltetra
  USE wvfct,      ONLY : nbnd, et
  USE lsda_mod,   ONLY : nspin
  USE control_dos, ONLY : legauss, sigmae, deltae, ndose, save_ndos
  USE noncollin_module, ONLY: noncolin
  USE control_bands, ONLY : emax_input, emin_input
  USE data_files, ONLY : fldos
  USE mp,         ONLY : mp_min, mp_max, mp_sum
  USE mp_pools,   ONLY : inter_pool_comm
  USE mp_images,  ONLY : my_image_id, root_image
  !
  IMPLICIT NONE
  !
  CHARACTER(LEN=256), EXTERNAL :: trimcheck
  !
  CHARACTER(len=256) :: fildos, outdir
  REAL(DP) :: emin, emax
  REAL(DP), ALLOCATABLE :: e(:), dosofe(:,:), dosint(:)
  REAL(DP) :: save_degauss
  INTEGER  :: save_ngauss
  LOGICAL  :: save_lgauss, save_ltetra
  INTEGER :: n, ndos, iu_dos
  !
  IF ( my_image_id /= root_image ) RETURN
  !
  CALL clean_pw(.TRUE.)
  CALL read_xml_file( )
  !
  save_degauss=degauss
  save_ngauss=ngauss
  save_ltetra=ltetra
  save_lgauss=lgauss
  IF (sigmae/=0.d0) THEN
     degauss=sigmae
     IF (legauss) ngauss = 0
     WRITE( stdout,'(/5x,"Gaussian broadening (read from input): ",&
             &        "ngauss,degauss=",i4,f12.6," Ry", /)') ngauss, degauss
     ltetra=.false.
     lgauss=.true.
  ELSEIF (ltetra) THEN
     WRITE( stdout,'(/5x,"Tetrahedra used"/)')
  ELSEIF (legauss) THEN
     ngauss=0
     WRITE( stdout,'(/5x,"Gaussian broadening (read from file): ",&
          &        "ngauss,degauss=",i4,f12.6, " Ry",/)') ngauss,degauss
     ltetra=.FALSE.
     lgauss=.TRUE.
  ELSE
     ngauss=0
     degauss=0.01_DP
     WRITE( stdout,'(/5x,"Broadening (default values): ",&
             &        "ngauss,degauss=",i4,f12.6," Ry"/)') ngauss,degauss
  ENDIF
  !
  ! find min and max energy for plot (band extrema if not set)
  !
  IF ( emin_input == 0.0_DP ) THEN
     emin = MINVAL ( et(1, 1:nks) )
     CALL mp_min(emin, inter_pool_comm)
     IF ( degauss > 0.0_dp ) emin = emin - 3.0_dp * degauss
  ELSE
     emin = emin_input/rytoev
  END IF
  IF ( emax_input  == 0.0_DP ) THEN
     emax = MINVAL ( et(nbnd, 1:nks) )
     CALL mp_max(emax, inter_pool_comm)
     IF ( degauss > 0.0_dp ) emax = emax + 3.0_dp * degauss
  ELSE 
     emax = emax_input/rytoev
  END IF
  !
  IF (deltae > 0.0_DP) THEN
     ndos = nint ( (emax - emin) / deltae)
  ELSEIF (ndose > 0 ) THEN
     ndos = ndose
     deltae = (emax-emin) / (ndos - 1)
  ELSE
     deltae=0.01_DP
     ndos = nint ( (emax - emin) / deltae)
  ENDIF

  save_ndos=ndos

  ALLOCATE(e(ndos))
  ALLOCATE(dosofe(2,ndos))
  ALLOCATE(dosint(ndos))

  dosofe=0.0_DP
  dosint = 0.d0
  !
  DO n= 1, ndos
     e(n) = emin + (n - 1) * deltae
     IF (ltetra) THEN
        CALL dos_t(et,nspin,nbnd,nks,ntetra,tetra,e(n),dosofe(1,n))
     ELSE
        CALL dos_g(et,nspin,nbnd,nks,wk,degauss,ngauss,e(n),dosofe(1,n))
     ENDIF
  ENDDO
!
!  sum over pools
!
  CALL mp_sum(dosofe, inter_pool_comm)
  
  IF (nspin==1.or.nspin==4) THEN
     dosint(1) = dosofe (1,1) * deltae
  ELSE
     dosint(1) = (dosofe (1,1) + dosofe (2,1) ) * deltae
  ENDIF

  DO n=2,ndos
     IF (nspin==1.or.nspin==4) THEN
        dosint(n) = dosint(n-1) + dosofe (1,n) * deltae
     ELSE
        dosint(n) = dosint(n-1) + (dosofe (1,n) + dosofe (2,n) ) * deltae
     ENDIF
  END DO
  !
  !  Write the results on file
  !
  IF ( ionode ) THEN
     IF ( fldos == ' ' ) THEN
        fildos = trim(prefix)//'.dos'
     ELSE
        fildos=TRIM(fldos)
     ENDIF

     iu_dos=2
     OPEN (unit=iu_dos, file=fildos, status='unknown', form='formatted')

     IF (nspin==1 .OR. nspin==4) THEN
        WRITE(iu_dos,'("#  E (eV)   dos(E)     Int dos(E)")')
     ELSE
        WRITE(iu_dos,'("#  E (eV)   dosup(E)     dosdw(E)   Int dos(E)")')
     ENDIF

     DO n= 1, ndos
        IF (nspin==1 .OR. nspin==4) THEN
           WRITE (iu_dos, '(f10.6,2e16.8)') e(n)*rytoev, dosofe(1,n)/rytoev,&
                                            dosint(n)
        ELSE
           WRITE (iu_dos, '(f10.3,3e16.8)') e(n)*rytoev, dosofe(:,n)/rytoev,&
                                            dosint(n)
        ENDIF
     ENDDO

     CLOSE (unit = iu_dos)
     !
  ENDIF

  degauss=save_degauss
  ngauss=save_ngauss
  ltetra=save_ltetra
  lgauss=save_lgauss

  DEALLOCATE(dosint)
  DEALLOCATE(dosofe)
  DEALLOCATE(e)
  !
  RETURN
END SUBROUTINE dos_sub
